#!/usr/bin/python

__author__ =  'Mohamed A. Bamakhrama'
__description__ = 'Channel Model for CSDF Graph'

import sys
import math

class ChannelModel:
    """ChannelModel class
    Represents a communication channel in a CSDF graph
    """

    def __init__(self, channel_id, name, source, destination, production_sequence, consumption_sequence):
        """Constructor"""
        self.name = name
        self.channel_id = channel_id
        self.source = source
        self.destination= destination
        self.buffer_size = sys.maxsize
        self.production_sequence = production_sequence
        self.consumption_sequence = consumption_sequence

    def set_source(self, source):
        """Get the source actor of the channel"""
        self.source = source

    def set_destination(self, destination):
        """Get the destination actor"""
        self.destination = destination

    def set_production_sequence(self, production_rate):
        """Set the production rates sequence"""
        self.production_rate = production_rate

    def set_consumption_sequence(self, consumption_sequence):
        """Set the consumption rates sequence"""
        self.consumption_sequence = consumption_sequence

    def set_buffer_size(self, buffer_size):
        """Set the channel buffer size"""
        assert buffer_size > 0
        self.buffer_size = buffer_size

    def set_name(self, name):
        """Set the name of the channel"""
        self.name = name

    @staticmethod
    def empty_firings(sequence):
        """Finds the index of the first zero rate in a prod/cons. sequence"""
        index = -1
        for i in range(0, len(sequence)):
            if sequence[i] > 0:
                index = i
                break
        return index

    def set_raw_inport(self, inport):
        self.raw_inport = inport

    def set_raw_outport(self, outport):
        self.raw_outport = outport

    def get_raw_inport(self):
        return self.raw_inport

    def get_raw_outport(self):
        return self.raw_outport

    def get_name(self):
        return self.name

    def get_channel_id(self):
        return self.channel_id

    def get_buffer_size(self):
        return self.buffer_size

    def get_source(self):
        return self.source

    def get_destination(self):
        return self.destination

    def get_production_sequence(self):
        return self.production_sequence

    def get_consumption_sequence(self):
        return self.consumption_sequence

    def __repr__(self):
        return "Channel %s : source = %s, destination = %s, production seq. = %s, consumption seq. = %s" % (self.name, self.source, self.destination, self.production_sequence, self.consumption_sequence)

