darts (Dataflow Analysis for Hard-Real-Time Scheduling)
================================================

darts is a framework for hard-real-time schedulability analysis of streaming applications modeled as dataflow graphs. Given a set of graphs, where each graph corresponds to an applications, it performs the following:

* Constructing a complete task-set of asynchronous periodic tasks for each application
* Computing the maximum throughput and minimum latency of each application under strictly periodic scheduling
* Deriving the minimum number of processors needed to schedule the set of applications on homogeneous systems by different scheduling algorithms (e.g., Partitioned EDF and optimal)
* Generating platform and mapping specifications which can be used as input to the Daedalus^RT design flow

Currently, darts supports only a single model of dataflow graphs which is acyclic Cyclo-Static Dataflow (CSDF) graphs.

Task-set Construction
---------------------
Given an acyclic CSDF graph, it derives an asynchronous set of periodic tasks. The deadline setting is customizable using a deadline factor (see below). The task-set derivation is implemented in ACSDFModel.py and can be invoked as follows:

    ACSDFModel.py [-h] (--minimum factor | --sufficient) graph

where:
`--minimum` computes the minimum start times and buffer sizes using the given deadline factor. The factor value must be in the range [0,1]. The deadline is computed as follows:

    D_i = C_i + factor*(T_i - C_i)

where `D_i` is the deadline of task `f_i`, `T_i` is the period of `f_i`, and `C_i` is the WCET of `f_i`. Therefore, a factor value equal to 1.0 is equivalent to the implicit-deadline model. On the other hand, `--sufficient` computes sufficient start times and buffer sizes.

`graph` is the path to the .gph file containing the CSDF graph. The .gph file format is explained below under "Graph Formats". `ACSDFModel.py` produces its output to `stdout`. So it might be useful for the user to redirect the output to a file. The full list of options of ACSDFModel.py can be shown by executing it and passing `-h` as an option.

Computing the throughput and latency
------------------------------------
The minimum latency and maxmimum throughput are automatically computed during the task-set derivation. The latency is shown next to `Graph maximum latency` field. The throughput is the reciprocal of the output actor(s) period(s).

Platform and Mapping Specifications
-----------------------------------
The platform and mapping specifications can be found under different scheduling algorithms. The supported mapping are listed in the supported_mappings variable in PlatformGenerator.py which determins the platform and mapping specifications. It can be invoked as follows:
    
    PlatformGenerator.py [-h] -o directory [-c] -m algorithm -a scaling-factor max-token-size graph 

where:

required arguments:
  -o directory          The output directory where the platform and mapping
                        specifications will be generated
  -m algorithm          The scheduling and mapping algorithm
  -a scaling-factor max-token-size graph
                        Specifies an application that will be processed.
                        scaling-factor is an integer >=1 that is used to scale
                        the minimum period. max-token-size is the maximum
                        token size (in 32-bit words). graph is the path to the
                        .gph file containing the CSDF model of the application
                        Multiple applications can be specified by passing `-a` multiple times. 
                        For example:
                        
                            -a 1 10 app1.gph -a 2 1 app2.gph -a 10 1 app3.gph


optional arguments:
  -h, --help            show a help message and exit
  -c                    Output the time values in cycles instead of OS clock ticks


`PlatformGenerator.py` produces several files. The most important two are: platform 
specifications (`platform.pla`) and mapping specifications (`mapping.map`)


Graph Formats
-------------

darts accepts as an input the graphs in StreamIt Modified Format (GPH Format) produced by [pntools](https://daedalus.liacs.nl/pntools/). An example of the GPH format for an SDF graph consisting of two actors (producer and consumer) is shown below:

    name:prod_cons
    type:csdf
    node_number:2
    node:
	    id:0
	    name:ND_0
	    function:producer
	    length:1
	    wcet:5
	    port_number:1
	    port:
		    type:out
		    id:1
		    rate:1
    node:
	    id:1
	    name:ND_1
	    function:consumer
	    length:1
	    wcet:2
	    port_number:1
	    port:
		    type:in
		    id:1
		    rate:1
    edge_number:1
    edge:
        id:0
        name:ED_0
        src:0 1
        dst:1 1	    

The indentation in the .gph should be done with the tab character ('\t'). In order to facilitate conversion between different formats, the user can use the `CSDFParser.py` to convert from .gph format into the the XML format required by [SDF For Free](http://www.es.ele.tue.nl/sdf3/). This can be done as follows:

    ./CSDFParser.py -s < file.gph > file.xml

Bugs/Questions
---------------
In case of bugs/questions, please contact [Mohamed Bamakhrama](mohamed@liacs.nl)

LICENSE
-------
This tool is released under the BSD license shown in LICENSE.md

