#!/usr/bin/python

import math
import os
import sys
import subprocess

sys.path.append("..")

from ACSDFModel import ACSDFModel
import Utilities


def benchmark(benchmarks_dir, sched_type="EDF"):
    if not os.path.isdir(benchmarks_dir):
        sys.stderr.write("Error! %s is not a directory!\n" % benchmarks_dir)
        sys.exit(-1)

    if sched_type != "EDF" and sched_type != "RM":
        sys.stderr.write("Error! Only EDF and RM are supported\n")
        sys.exit(-1)

    os.chdir(benchmarks_dir)

    child = subprocess.Popen(["ls -1 *.gph"], shell=True, stdout=subprocess.PIPE)
    gph_files = child.stdout.readlines()

    for gp in gph_files:
        gph = gp.rstrip('\n')
        sys.stdout.write(gph + ": \n")

        scaling_factor = 1
        minimum_sink_period = 0

        g = ACSDFModel.load_graph_from_file(gph, scaling_factor)

        # TODO: I assume that the sink actor is the first one in the last level
        g_levels = g.find_levels()
        aa = next(iter(g_levels[-1]))
        ac = g.get_actor(aa.get_actor_id())
        minimum_sink_period = ac.get_period()

        if sched_type == "EDF":
            UTILIZATION_BOUND = 1.0
        if sched_type == "RM":
            UTILIZATION_BOUND = Utilities.RM_exact_bound(g.get_num_of_actors())

        while g.get_utilization() > UTILIZATION_BOUND:
            scaling_factor += 1
            g = ACSDFModel.load_graph_from_file(gph, scaling_factor)

        # Find the start time and buffer size once we reached the required utilization        
        g.calc_start_time_buffer_vector(ACSDFModel.MIN, 1)

        sys.stdout.write("\tUtilization = %s\n" % g.get_utilization())
        g_levels = g.find_levels()

        # TODO: I assume that the sink actor is the first one in the last level
        aa = next(iter(g_levels[-1]))
        ac = g.get_actor(aa.get_actor_id())
        sys.stdout.write("\tActual period = %s\n" % ac.get_period())
        sys.stdout.write("\tMinimum period = %s\n" % minimum_sink_period)
        sys.stdout.write("\tScaling factor = %s\n" % scaling_factor)

        total_buffer_size = 0
        for c in g.get_channels():
            total_buffer_size += c.get_buffer_size()
        sys.stdout.write("\tTotal buffer size = %s\n" % total_buffer_size) 

if __name__ == "__main__":

    sched_type = "EDF"
    if len(sys.argv) > 1:
        if sys.argv[1] != "EDF" and sys.argv[1] != "RM":
            sys.stderr.write("Error! Only EDF and RM are supported\n")
            sys.exit(-1)

        if sys.argv[1] == "RM":
            sched_type = "RM"

    benchmarks_dir = "benchmarks/gph/"
    benchmark(benchmarks_dir, sched_type)

